@@ -3,6 +3,7 @@
 //   Mikael Hallendal <micke@imendio.com>
 //
 // (C) 2004 Imendio AB
+// (C) 2008 Nuanti Ltd.
 //
 
 using GConf;
@@ -17,10 +18,35 @@ using System.Collections;
 using System.IO;
 using System.Text;
 using System.Runtime.InteropServices;
+using WebKit;
+#if ENABLE_DBUS
+using NDesk.DBus;
+using org.freedesktop.DBus;
+#endif
 
 namespace Imendio.Blam {
 
-    public class Application : Program {
+#if ENABLE_DBUS
+    [NDesk.DBus.Interface("org.gnome.feed.Reader")]
+#endif
+    public interface IFeedReader {
+        bool Subscribe(string url);
+        void SetOnline(bool value);
+    }
+
+#if ENABLE_DBUS
+    [NDesk.DBus.Interface("org.gnome.Blam.Reader")]
+#endif
+    public interface IBlamReader {
+        void ShowWindow();
+    }
+
+    public class Application : Program, IBlamReader, IFeedReader {
+
+#if ENABLE_DBUS
+        static string bus_name = "org.gnome.feed.Reader";
+        static ObjectPath obj_path = new ObjectPath("/org/gnome/feed/Reader");
+#endif
 
 	[DllImport("libc")]
 	private static extern int prctl(int option, byte [] arg2, ulong arg3, 
@@ -47,7 +73,7 @@ namespace Imendio.Blam {
         [Widget] Gtk.MenuItem       editEntryKeywordsMenuItem = null;
         [Widget] Gtk.MenuItem       nextUnreadMenuItem = null;
 
-        //[Widget] Gtk.MenuItem       printMenuItem;
+        [Widget] Gtk.MenuItem       printMenuItem = null;
 
         private Gtk.FileChooserDialog   exportFileDialog;
 
@@ -61,8 +87,6 @@ namespace Imendio.Blam {
 
         public static string BaseDir;
 
-        private MessageConnection messageConn;
-
         private AddChannelDialog  addChannelDialog;
         private ChannelDialog     channelDialog;
         private PreferencesDialog preferencesDialog;
@@ -72,8 +96,8 @@ namespace Imendio.Blam {
 
         private uint              mAutoRefreshId;
 
+        Gnome.Client client = null;
 
-        private MessageConnection.MessageReceivedHandler mIpcHandler;
 
         enum TargetType {
             String,
@@ -112,19 +136,15 @@ namespace Imendio.Blam {
 
         public Application (string[] args, params object[] props) : base ("Blam", Defines.VERSION, Modules.UI, args, props)
         {
-            messageConn = new MessageConnection ("Blam");
-            if (!messageConn.IsServer) {
-                messageConn.Send ("ShowWindow");
-                messageConn.Close ();
-                Gdk.Global.NotifyStartupComplete ();
-                Environment.Exit (0);
-            }
+            SetupDBus();
 
-            mIpcHandler = new MessageConnection.MessageReceivedHandler (MessageReceivedCb) ;
-            messageConn.SetCallback (mIpcHandler);
+            client = new Gnome.Client();
+            client.Connect();
+            client.SaveYourself += SaveMyself;
+            client.Die += QuitActivated;
+            client.RestartStyle = RestartStyle.IfRunning;
 
             Proxy.InitProxy ();
-            Utils.GeckoInit ();
 
             Catalog.Init ("blam", Defines.GNOME_LOCALE_DIR);
 
@@ -148,7 +168,9 @@ namespace Imendio.Blam {
 
             GLib.Idle.Add (new GLib.IdleHandler (IdleRefreshAll));
 
-            SetupDBus ();
+            if(Conf.Get(Preference.REFRESH_AT_START, false) == true){
+                mCollection.RefreshAll();
+            }
         }
 
         private void PrepareGUI()
@@ -170,22 +192,23 @@ namespace Imendio.Blam {
 
             Frame f = new Frame ();
             f.Shadow = ShadowType.In;
-            f.Add (itemView.Widget);
+            f.Add (itemView);
             itemPaned.Add2 (f);
             f.Show ();
             itemView.OnUrl += OnUrl;
+            //itemView.Widget.DomMouseClick += new Gecko.DomMouseClickHandler(OnButtonPressEvent);
 
             itemList = new ItemList(itemView);
             ((Container)itemListSw).Child = itemList;
 
             itemList.ItemSelected += ItemSelected;
 
-            trayIcon = new TrayIcon (Catalog.GetString ("Imendio Blam News Reader"));
+            trayIcon = new TrayIcon (Catalog.GetString ("Blam News Reader"));
             trayIcon.ButtonPressEvent += TrayIconButtonPressCb;
             channelsLabelText = channelsLabel.Text;
             UpdateTotalNumberOfUnread ();
 
-            //printMenuItem.Sensitive = false;
+            printMenuItem.Sensitive = false;
             SensitizeChannelMenuItems(false);
 
             // Setup drag-n-drop
@@ -199,6 +222,13 @@ namespace Imendio.Blam {
 
             mainWindow.ShowAll ();
 
+            int ShowItemList = Conf.Get("ui/show_item_list", 1);
+            if(ShowItemList == 1){
+                itemPaned.Child1.Visible = true;
+            } else {
+                itemPaned.Child1.Visible = false;
+            }
+
             channelDialog = new ChannelDialog (this);
             addChannelDialog = new AddChannelDialog (this);
             preferencesDialog = new PreferencesDialog (this.Window);
@@ -208,14 +238,70 @@ namespace Imendio.Blam {
 
         }
 
+        public void ToggleItemList()
+        {
+            if(itemPaned.Child1.Visible){
+                itemPaned.Child1.Visible = false;
+                Conf.Set("ui/show_item_list", 0);
+            } else {
+                itemPaned.Child1.Visible = true;
+                Conf.Set("ui/show_item_list", 1);
+            }
+        }
+
+        // Exposed by D-Bus
+        public void ShowWindow()
+        {
+            this.mainWindow.Present();
+        }
+
+        // Exposed by D-Bus
+        public bool Subscribe(string url)
+        {
+            Channel channel = new Channel();
+            channel.Url = url;
+            
+            CCollection.Add(channel);
+            
+            return true;
+        }
+
+        // Exposed by D-Bus
+        public void SetOnline(bool value)
+        {
+          // This is here to match Liferea.
+          // Not yet implemented.
+        }
+
 #if ENABLE_DBUS
         private void SetupDBus()
         {
             try{
-                DBusMgr dbusmgr = new DBusMgr(this);
-                dbusmgr.Register();
-            } catch (Exception e){
-                Console.WriteLine("Unable to start DBus interface: {0}", e.Message);
+                BusG.Init();
+                Bus bus = Bus.Session;
+
+                if (bus.RequestName(bus_name) == RequestNameReply.PrimaryOwner) {
+                    bus.Register(obj_path, this);
+                } else {
+                    IBlamReader reader = bus.GetObject<IBlamReader>(bus_name, obj_path);
+                    reader.ShowWindow();
+                    Gdk.Global.NotifyStartupComplete ();
+                    Environment.Exit(0);
+                }
+
+            } catch (Exception e) {
+                Console.Error.WriteLine("Unable to start D-Bus interface: {0}", e.Message);
+            }
+        }
+
+        private void StopDBus()
+        {
+            try {
+                Bus bus = Bus.Session;
+                bus.Unregister(obj_path);
+                bus.ReleaseName(bus_name);
+            } catch (Exception e) {
+                Console.Error.WriteLine("Unable to stop D-Bus interface: {0}", e.Message);
             }
         }
 #else
@@ -223,8 +309,50 @@ namespace Imendio.Blam {
         {
             // Do nothing
         }
+
+        private void StopDBus()
+        {
+            // Stub
+        }
 #endif
 
+#if FALSE
+        private void OnButtonPressEvent(object o, Gecko.DomMouseClickArgs args)
+        {
+            int button = Utils.GeckoGetMouseButton(args.DomEvent);
+
+            if(button == 2){ // Right button
+                Gtk.Menu popup = new Gtk.Menu();
+
+                if(itemView.Widget.LinkMessage != null && itemView.Widget.LinkMessage != string.Empty){ // Make sure we're on a link.
+                    GtkUtil.AppendMenuItem(popup,
+                                       Catalog.GetString("_Open link in browser"),
+                                       new EventHandler(OpenLink), true);
+                    GtkUtil.AppendMenuItem(popup,
+                                       Catalog.GetString("_Copy link location"),
+                                       "gtk-copy", new EventHandler(CopyLink), true);
+
+                    popup.Popup();
+                }
+            }
+
+            args.RetVal = 0;
+        }
+#endif
+
+        void CopyLink(object o, EventArgs args)
+        {
+            Clipboard clip = Clipboard.Get(Gdk.Atom.Intern("CLIPBOARD", true));
+						// FIXME: WebKit
+            //clip.Text = itemView.Widget.LinkMessage;
+        }
+
+        void OpenLink(object o, EventArgs args)
+        {
+						// FIXME: WebKit
+            //Gnome.Url.Show(itemView.Widget.LinkMessage);
+        }
+
         private void ChannelSelected(Channel channel)
         {
             if (channel == null) {
@@ -252,7 +380,7 @@ namespace Imendio.Blam {
         private void ItemSelected(Imendio.Blam.Item item)
         {
             itemView.CurrentItem = item;
-            //printMenuItem.Sensitive = true;
+            printMenuItem.Sensitive = true;
         }
 
         public void MarkEntryAsUnreadActivated (object obj, EventArgs args)
@@ -326,15 +454,22 @@ namespace Imendio.Blam {
                 break;
             case (Gdk.Key.Escape):
                 mainWindow.Hide ();
+                trayIcon.Show();
+                break;
+            case (Gdk.Key.I):
+            case (Gdk.Key.i):
+                ToggleItemList();
                 break;
             }
         }
 
-        /*private void PrintActivated(object obj, EventArgs args)
-    {
-        Blam.PrintDialog dialog = new Blam.PrintDialog(itemView);
-        dialog.Run();
-    }*/
+        public void PrintActivated(object obj, EventArgs args)
+        {
+            //if (!itemView.PageLoaded)
+            //  return;
+
+            itemView.Widget.ExecuteScript ("print();");
+        }
 
         public void ImportOpmlActivated (object obj, EventArgs args)
         {
@@ -423,12 +558,19 @@ namespace Imendio.Blam {
             trayIcon.Show();
             args.RetVal = true;
         }
+        
+        public void SaveMyself(object obj, SaveYourselfArgs args)
+        {
+            string[] argv = {"blam"};
+
+            client.SetRestartCommand(1, argv);
+        }
 
         public void QuitActivated(object obj, EventArgs args)
         {
+            StopDBus();
             SaveWindowState();
             mainWindow.Hide();
-
             mCollection.SaveToFile ();
             mCollection.StopAllThreads();
 
@@ -437,8 +579,7 @@ namespace Imendio.Blam {
 
         public void CopyActivated (object obj, EventArgs args)
         {
-            // FIXME: Check how to do this
-            //itemView.Copy ();
+            itemView.Widget.CopyClipboard();
         }
 
         public void PreferencesActivated (object obj, EventArgs args)
@@ -480,6 +621,13 @@ namespace Imendio.Blam {
             MarkChannelAsReadActivated (channelList.GetSelected ());
         }
 
+        public void MarkAllAsReadActivated(object o, EventArgs args)
+        {
+            foreach(Channel channel in CCollection.Channels){
+                MarkChannelAsReadActivated(channel);
+         }
+        }
+
         public void RemoveChannelActivated(Channel channel)
         {
             if (channel != null) {
@@ -713,16 +861,6 @@ namespace Imendio.Blam {
             }
         }
 
-        private void MessageReceivedCb (string message, IntPtr userData)
-        {
-            if (!message.Equals ("ShowWindow")) {
-                return;
-            }
-
-            mainWindow.Present ();
-            //mCollection.RefreshAll ();
-        }
-
 	public static void SetProcessName(string name)
 	{
 	    if(prctl(15 /* PR_SET_NAME */, Encoding.ASCII.GetBytes(name + "\0"), 
@@ -734,7 +872,11 @@ namespace Imendio.Blam {
 
         public static void Main(string[] args)
         {
-            SetProcessName("blam");
+            try {
+                SetProcessName("blam");
+						} catch {
+						}
+
             TheApp = new Application (args);
             TheApp.Run ();
             // new Application(args).Run();
