@@ -5,7 +5,6 @@
 // (C) 2004 Imendio HB
 // 
 
-using Rss;
 using System;
 using System.Collections;
 using System.IO;
@@ -125,7 +124,7 @@ namespace Imendio.Blam {
 		    serializer.Serialize (writer, this);
 		    writer.Close();
 		} catch (Exception) {
-		    System.Console.WriteLine ("Failed to save to temporary file");
+		    Console.Error.WriteLine ("Failed to save to temporary file");
 		    return;
 		}
 
@@ -137,7 +136,7 @@ namespace Imendio.Blam {
 
 		    File.Move (tmpFile, this.FileName);
 		} catch (Exception e) {
-		    System.Console.WriteLine ("Move failed: " + e.Message);
+		    Console.Error.WriteLine ("Move failed: " + e.Message);
 		    return;
 		}
 		
